from harvest.common.url_tools import update_url_with_parameters
from harvest.common.ordereddict import OrderedDict #while we wait for Python 2.7 :)

class FilterContainer(object): #abstract
    """
    A class that contains Filter objects, which can be retrieved with the
    find method.
    
    The added Filter objects are referred to as "children." They are expected
    to exist for the entire life of the container.
    """
    
    def __init__(self, filters_list): 
        self.filters_dict = OrderedDict() #refers to Filter objects by their unique IDs
        self.add_filters(filters_list)
    
    def add_filters(self, filters_list): #final
        """
        Adds a list of filters to be children of this one and informs
        each child that it belongs to this container.
        @param filter_list: a list of Filter objects
        """
        for child in list(filters_list):
            self.filters_dict[child.get_id()] = child
            child.set_container(self)
    
    def find(self, full_name): #final
        """
        Finds a filter inside this object or one of its children, based
        on that filter's full name in the format container-child.
        @param full_name: an object's full name
        @return: the object described by full_name, or None
        """
        result = None
        name_bits = full_name.split('.',1)
        
        #the first bit is the one this instance stores
        if name_bits[0] in self.filters_dict:
            result = self.filters_dict[name_bits[0]]
        
        if isinstance(result, FilterContainer) and len(name_bits)>1:
            result = result.find(name_bits[1]) #send remaining bits to child
        
        #we have reached the end of the list. Return the result (which may be None)
        return result



class FilterSystem(FilterContainer):
    """
    This is the single all-knowing root object that should contain all
    other filters for an application. From this object it is possible to
    find a filter using its full name, which is the same name used to
    serialize a filter's state in an HTTP query string.
    
    Before an instance of this object can be safely used,
    update_from_http should be called with the current HttpRequest.
    """
    
    def __init__(self, filters_list, default_parameters = dict()):
        FilterContainer.__init__(self, filters_list)
        self.request = None #current http request
        self.default_parameters = default_parameters
    
    def update_from_http(self, request): #final
        """
        Call this method to update the state of all filters based on an
        HttpRequest object. The request object will be stored for other uses.
        @param request: current HttpRequest object
        """
        self.request = request
        #this contains parameters that aren't relevant to the FilterSystem
        self.set_parameters(request.GET)
    
    def set_parameters(self, querydict): #final
        """
        Updates the state of all filters based on given parameters.
        @param querydict: dictionary of parameters from request.GET
        """
        new_params = querydict.copy()
        
        #apply default values for keys that have not been set
        for key in self.default_parameters:
            value = self.default_parameters[key]
            if isinstance(value,list):
                new_params.setlistdefault(key, value)
            else:
                new_params.setdefault(key, value)
        
        for key in new_params:
            filter_object = self.find(key)
            if filter_object:
                filter_object.set_value(new_params.getlist(key))
    
    def get_url_with_parameters(self, parameters): #final
        """
        Returns a new URL where the given parameters will be applied.
        To generate parameters, see Filter.serialize.
        @param parameters: a dictionary of new parameters
        @return: the current url with the given parameters added to the query string
        """
        return update_url_with_parameters(self.request, parameters)

