# -*- coding: utf-8 -*-
from django.contrib.auth.decorators import login_required
from django.utils.translation import ugettext as _
from django.shortcuts import get_object_or_404
from django.shortcuts import render_to_response as render
from django.http import HttpResponseRedirect

from django.template import RequestContext

import models
import forms

from filters import HarvestFilters
from wrappers import PackageWrapper, PackageListWrapper

def _create_packages_list(request, filters_pkg, filters_opp):
    # XXX: rockstar: Eep! We shouldn't be storing the None as a string.  We
    # should re-think this model relationship.
    #sourcepackages_list = models.SourcePackage.objects.exclude(name='None')
    
    sourcepackages_list = models.SourcePackage.objects.distinct()
    sourcepackages_list = filters_pkg.process_queryset(sourcepackages_list)
    
    #opportunities_list is filtered right away to only check opportunities belonging to selected packages
    opportunities_list = models.Opportunity.objects.distinct().filter(sourcepackage__in=sourcepackages_list)
    opportunities_list = filters_opp.process_queryset(opportunities_list)
    
    #TODO: need to filter out opportunities with valid=False again
    #TODO: would it be more efficient to group opportunities by their sourcepackages first, then run filters_opp.process_queryset() for each of those groups?
    
    pkg_list_wrapper = PackageListWrapper(request, sourcepackages_list, opportunities_list)
    
    return pkg_list_wrapper



def filter(request):
    filters = HarvestFilters()
    filters.update_from_http(request)
    filters_pkg = filters.find('pkg')
    filters_opp = filters.find('opp')
    
    pkg_list_wrapper = _create_packages_list(request, filters_pkg, filters_opp)
    
    context = {
        'packages_list' : pkg_list_wrapper,
        'filters_pkg' : filters_pkg,
        'filters_opp' : filters_opp
    }
    
    if len(request.GET) == 0:
        #an empty querystring indicates this has been opened for the first time
        context['show_intro_message'] = True
    
    return render(
        'opportunities/filter.html',
        context,
        context_instance=RequestContext(request))

def single_package(request, package_name):
    package = get_object_or_404(models.SourcePackage, name=package_name)
    
    package_wrapper = PackageWrapper(request, package, visible_opportunities = package.opportunity_set)
    
    context = {
        'package': package_wrapper
    }
    
    return render(
        'opportunities/single_package.html',
        context,
        context_instance=RequestContext(request))

@login_required
def opportunity_edit(request, opportunity_id, template='opportunities/opportunity_edit.html'):
    opportunity = get_object_or_404(models.Opportunity, id=opportunity_id)
    if request.method == "POST":
        form = forms.OpportunityForm(data=request.POST, instance=opportunity)
        if form.is_valid():
            if form.cleaned_data["reviewed"] != opportunity.reviewed:
                models.log_action(request.user, action="marked as reviewed", 
                                  opportunity=opportunity)
            if form.cleaned_data["applied"] != opportunity.applied:
                models.log_action(request.user, action="marked as applied", 
                                  opportunity=opportunity)
            if form.cleaned_data["experience"] != opportunity.experience:
                models.log_action(request.user, 
                                  action="changed experience to: %s" % form.cleaned_data["experience"],
                                  opportunity=opportunity)
            form.save()
            
            #add a new note if input by the user
            if form.cleaned_data["new_note"].strip() != '':
                note_text = form.cleaned_data["new_note"]
                
                note_log_text = note_text
                if len(note_log_text) > 160:
                    note_log_text = note_log_text[:160]+u"\u2026"
                models.log_action(request.user,
                                  action="added note: '%s'" % note_log_text,
                                  opportunity=opportunity)
                
                note = models.Note(opportunity=opportunity, author=request.user, text=note_text)
                note.save()
            
            return HttpResponseRedirect(request.POST["next"])
        else:
            request.user.message_set.create(message=_('Opportunity details could not be saved.'))
    else:
        form = forms.OpportunityForm(instance=opportunity)
    
    next = '/'
    if 'next' in request.GET: next = request.GET['next']
    
    return render(template, 
                    {'form': form, 
                     'opportunity':opportunity,
                     'user':request.user, 
                     'next': next,
                    }, RequestContext(request))



def xhr_filter_results(request):
    filters = HarvestFilters()
    filters.update_from_http(request)
    
    packages_list = _create_packages_list(request, filters.find('pkg'), filters.find('opp')) 
    
    context = {
        'packages_list': packages_list
    }

    return render(
        'opportunities/xhr/filter_results.html',
        context,
        context_instance=RequestContext(request))

def xhr_package_details(request, package_id):
    filters = HarvestFilters()
    filters.update_from_http(request)
    
    package = get_object_or_404(models.SourcePackage, id=package_id)
    
    opportunities_list = filters.find('opp').process_queryset(package.opportunity_set).all()
    
    package_wrapper = PackageWrapper(request, package, visible_opportunities = opportunities_list, expanded = True)
    
    context = {
        'package': package_wrapper
    }
    
    return render(
        'opportunities/xhr/package_details.html',
        context,
        context_instance=RequestContext(request))

def xhr_opportunity_li(request, opportunity_id):
    opportunity = get_object_or_404(models.Opportunity, id=opportunity_id)
    
    context = {
        'opportunity': opportunity
    }
    
    return render(
        'opportunities/xhr/opportunity_outer_li.html',
        context,
        context_instance=RequestContext(request))

def xhr_opportunity_edit(request, opportunity_id):
    return opportunity_edit(
        request,
        opportunity_id,
        template='opportunities/xhr/opportunity_edit.html')

